﻿#================================================================
#
# 朝監視用BOM情報取得（最終監視結果、ヒストリーログ）
#
# [スクリプト説明]
#   朝監視レポート用にBOMの監視情報を取得し、XML形式でデータ出力を行う。
#   取得する監視情報は以下。
#   ・Windowsサービス監視（サービス監視名称、直近の監視値を取得）
#   ・BOMヒストリーログ（監視/アクションログから対象キーワード"@"が含まれるログを取得）
#
# [パラメーター説明]
#   Param: BOMインスタンス名
#   Param: 出力XMLパス
#
# [終了コード]
#   0: 正常
#   1: インストールパス取得エラー
#
# [注意事項]
#   ・BOM for Windows Ver.8.0にのみ対応
#   ・Windows Server 2016以降に対応
#
#================================================================

#----------------------------------------------------------------
# パラメーター定義
#----------------------------------------------------------------
Param (
    [parameter(mandatory=$true)][String] $prm_bom_instance,
    [parameter(mandatory=$true)][String] $prm_out_xml_path
)

#----------------------------------------------------------------
# 外部ファイル読み込み
#----------------------------------------------------------------
$current_dir = (Convert-Path .) 
Set-Location -LiteralPath $PSScriptRoot
.".\ClassBomData.ps1"
Set-Location -LiteralPath $current_dir

#----------------------------------------------------------------
# 定数定義
#----------------------------------------------------------------
Set-Variable -name EXIT_SUCCESS -value 0 -Option Constant
Set-Variable -name EXIT_ERROR_INSTALLPATH_NOT_FOUND -value 1 -Option Constant
Set-Variable -name EXIT_ERROR_BOM_MONITOR -value 2 -Option Constant
Set-Variable -name EXIT_ERROR_BOM_HISTORYLOG -value 3 -Option Constant

#----------------------------------------------------------------
# 関数定義
#----------------------------------------------------------------
# アプリケーションインストール場所を取得
function GetApplicationInstallPath($app_name)
{
    $inst_path = ""
    try {
        $inst_path = Get-ItemProperty "HKLM:\Software\Microsoft\Windows\CurrentVersion\Uninstall\*" |
                       Where-Object { $_.DisplayName -eq $app_name } |
                       Select-Object -ExpandProperty InstallLocation
        if (-not $inst_path) {
            throw "アプリケーションがインストールされていないか、インストールパスが見つかりませんでした。"
        }
    } catch {
        $inst_path = ""
        Write-Host "エラー: $_" 
    }

    return $inst_path
}

# レジストリ取得
function GetRegistryValue($reg_key, $entry)
{
    $value = ""
    try {
        $value = (Get-Item -LiteralPath $reg_key).GetValue($entry)
    } catch {
        $value = ""
        Write-Host "エラー: $_"
    }
    return $value
}

# BOMヒストリーログの出力文字列取得
function GetBomHistoryOutString($message)
{
    $logs = $message -split "`r`n"
    $id = ""
    foreach($log in $logs) {
        if ($log -match "^ID: (?<id>.*?)$") {
            $id = $Matches.id
            break;
        }
    }

    return $id, $logs[0]
}

#----------------------------------------------------------------
# メイン
#----------------------------------------------------------------

# BOM Binパス取得
$bom_bin_path = GetRegistryValue "HKLM:\SOFTWARE\SAY Technologies\BOMW8" "BinDir"
# BOM Dataパス取得
$bom_data_path = GetRegistryValue "HKLM:\SOFTWARE\SAY Technologies\BOMW8" "DataDir"
If ([string]::IsNullOrEmpty($bom_bin_path) -or [string]::IsNullOrEmpty($bom_data_path)) {
    Exit $EXIT_ERROR_INSTALLPATH_NOT_FOUND
}

# BOMから監視結果やヒストリーログを取得
$bom_data = New-Object BomData
if ($bom_data.GetBomXMLData($bom_data_path, $prm_bom_instance) -ne $True) {
    Exit $EXIT_ERROR_BOM_MONITOR
}
if ($bom_data.GetBomHistoryData($bom_bin_path, $prm_bom_instance, "", 1, 1) -ne $True) {
    Exit $EXIT_ERROR_BOM_HISTORYLOG
}

# XML作成
try
{
    $xmlDoc = New-Object System.Xml.XmlDocument

    # ルート要素を追加
    $rootElement = $xmlDoc.CreateElement("BomData")
    $xmlDoc.AppendChild($rootElement) > $null

    # インスタンスIDを追加
    $instance_result = $xmlDoc.CreateElement("BomInstance")
    $rootElement.AppendChild($instance_result) > $null
    $instance_id = $xmlDoc.CreateElement("InstanceId")
    $instance_id.InnerText = $prm_bom_instance
    $instance_result.AppendChild($instance_id) > $null

    # BOM監視結果を追加
    $monitor_result = $xmlDoc.CreateElement("BomMonitorResult")
    $rootElement.AppendChild($monitor_result) > $null
    foreach ($mon_data in $bom_data.MonitorDatas) {
        $result_item = $xmlDoc.CreateElement("MonitorResultItem")

        $item = $xmlDoc.CreateElement("GrpNo")
        $item.InnerText  = $mon_data.GrpNo
        $result_item.AppendChild($item) > $null

        $item = $xmlDoc.CreateElement("MonNo")
        $item.InnerText  = $mon_data.MonNo
        $result_item.AppendChild($item) > $null

        $item = $xmlDoc.CreateElement("ObjectName")
        $item.InnerText  = $mon_data.Setting.ObjectName
        $result_item.AppendChild($item) > $null

        $item = $xmlDoc.CreateElement("ValueName")
        $item.InnerText  = $mon_data.Setting.ValueName
        $result_item.AppendChild($item) > $null

        $item = $xmlDoc.CreateElement("Options")
        $item.InnerText  = $mon_data.Setting.Options
        $result_item.AppendChild($item) > $null

        $item = $xmlDoc.CreateElement("MonitorName")
        $item.InnerText  = $mon_data.Setting.Name
        $result_item.AppendChild($item) > $null

        if ($mon_data.LogDatas.Count -gt 0) {
            #$item = $xmlDoc.CreateElement("MonitorName")
            #$item.InnerText  = $mon_data.LogDatas[0].MonitorName
            #$result_item.AppendChild($item) > $null

            $item = $xmlDoc.CreateElement("Status")
            $item.InnerText  = $mon_data.LogDatas[0].Status
            $result_item.AppendChild($item) > $null

            $item = $xmlDoc.CreateElement("Value")
            $item.InnerText  = $mon_data.LogDatas[0].Value
            $result_item.AppendChild($item) > $null

            $item = $xmlDoc.CreateElement("ExecDateTime")
            $item.InnerText  = $mon_data.LogDatas[0].ExecDateTime
            $result_item.AppendChild($item) > $null

            $item = $xmlDoc.CreateElement("Code")
            $item.InnerText  = $mon_data.LogDatas[0].Code
            $result_item.AppendChild($item) > $null

            $item = $xmlDoc.CreateElement("Duration")
            $item.InnerText  = $mon_data.LogDatas[0].Duration
            $result_item.AppendChild($item) > $null
        }

        $monitor_result.AppendChild($result_item) > $null
    }

    # BOMヒストリーを追加
    $history = $xmlDoc.CreateElement("BomHistory")
    $rootElement.AppendChild($history) > $null
    foreach ($history_data in $bom_data.HistoryLogs) {
        $history_item = $xmlDoc.CreateElement("HistoryItem")

        $item = $xmlDoc.CreateElement("Type")
        $item.InnerText  = $history_data.Type
        $history_item.AppendChild($item) > $null

        $item = $xmlDoc.CreateElement("CreateDateTime")
        $item.InnerText  = $history_data.CreateDateTime
        $history_item.AppendChild($item) > $null

        $item = $xmlDoc.CreateElement("Category")
        $item.InnerText  = $history_data.Category
        $history_item.AppendChild($item) > $null

        $id, $log = GetBomHistoryOutString($history_data.Summary)
        $item = $xmlDoc.CreateElement("Id")
        $item.InnerText  = $id
        $history_item.AppendChild($item) > $null

        $item = $xmlDoc.CreateElement("Summary")
        $item.InnerText  = $log
        $history_item.AppendChild($item) > $null

        $history.AppendChild($history_item) > $null
    }
    $xmlDoc.Save($prm_out_xml_path)
    #$xmlDoc.SelectSingleNode("//BomMonitorResult").OuterXml | Out-File -FilePath ".\BomMonitorResult.xml" -Encoding UTF8
    #$xmlDoc.SelectSingleNode("//BomHistory").OuterXml | Out-File -FilePath ".\BomHistory.xml" -Encoding UTF8
} catch {
    Write-Host "例外: $_"
}

Exit $EXIT_SUCCESS